/* ----------------------------------------------------------------
 * Z88DK INTERFACE LIBRARY FOR THE BIFROST* ENGINE - RELEASE 1.2/L
 *
 * This file is released under the GPL v3 License
 *
 * If you use this interface library, you must load afterwards the
 * BIFROST* ENGINE and a multicolor tile set. For a detailed sample
 * see file "bifrostdem.c".
 *
 * Original version and further information is available at
 * http://www.worldofspectrum.org/infoseekid.cgi?id=0027405
 *
 * Copyleft (k) Einar Saukas, Timmy
 * Additional improvements provided by Alcoholics Anonymous
 * ----------------------------------------------------------------
 */
#ifndef __BIFROST_H__
#define __BIFROST_H__

extern unsigned char BIFROST_tilemap[81] @ 65281;

#define BIFROST_STATIC    128
#define BIFROST_DISABLED  255

// ----------------------------------------------------------------
// Activate multicolor rendering with BIFROST* ENGINE
// ----------------------------------------------------------------
#define BIFROST_start()  asm("call    64995  ; BIFROST_start()\n")

// ----------------------------------------------------------------
// Deactivate multicolor rendering with BIFROST* ENGINE
// ----------------------------------------------------------------
#define BIFROST_stop()  asm("call     65012  ; BIFROST_stop()\n")

// ----------------------------------------------------------------
// Place a multicolor tile index into the tile map. Add value
// BIFROSTSTATIC for static tile, otherwise it will be animated
//
// Parameters:
//     px: tile vertical position (0-8)
//     py: tile horizontal position (0-8)
//     tile: tile index (0-255)
//
// Obs: Also available as inline macro (for constant parameters)
// ----------------------------------------------------------------
extern void __LIB__ BIFROST_setTile(unsigned char px, unsigned char py, unsigned char tile);

#define M_BIFROST_SETTILE(px, py, tile)  BIFROST_tilemap[(px)*9+(py)] = (tile)

// ----------------------------------------------------------------
// Obtain a multicolor tile index from the tile map
//
// Parameters:
//     px: tile vertical position (0-8)
//     py: tile horizontal position (0-8)
//
// Returns:
//     Tile index currently stored in this position
//
// Obs: Also available as inline macro (for constant parameters)
// ----------------------------------------------------------------
extern unsigned char __LIB__ BIFROST_getTile(unsigned char px, unsigned char py);

#define M_BIFROST_GETTILE(px, py)   BIFROST_tilemap[(px)*9+(py)]

// ----------------------------------------------------------------
// Convert multicolor tile index into the equivalent animation group
//
// Parameters:
//     tile: tile index (0-255)
//
// Returns:
//     Animation group for animated tile, otherwise the same tile index
// ----------------------------------------------------------------
extern unsigned char __FASTCALL__ __LIB__ BIFROST_getAnimGroup(unsigned char tile);

// ----------------------------------------------------------------
// Locate memory address that stores the multicolor attribute of a
// certain screen position inside the multicolor area
//
// Parameters:
//     lin: pixel line (16-159)
//     col: char column (1-18)
//
// Returns:
//     Memory address of the multicolor attribute
// ----------------------------------------------------------------
extern unsigned char __LIB__ *BIFROST_findAttrH(unsigned char lin, unsigned char col);

// ----------------------------------------------------------------
// Reconfigure BIFROST* ENGINE to read tile images from another address
//
// Parameters:
//     addr: New tile images address
// ----------------------------------------------------------------
extern void __FASTCALL__ __LIB__ BIFROST_resetTileImages(unsigned int *addr);

// ----------------------------------------------------------------
// Reconfigure BIFROST* ENGINE to animate at 2 frames per second
// ----------------------------------------------------------------
#define BIFROST_resetAnimSlow()  *((unsigned char*)59035)=254

// ----------------------------------------------------------------
// Reconfigure BIFROST* ENGINE to animate at 4 frames per second
// ----------------------------------------------------------------
#define BIFROST_resetAnimFast()  *((unsigned char*)59035)=198

// ----------------------------------------------------------------
// Reconfigure BIFROST* ENGINE to use 2 frames per animation group
// ----------------------------------------------------------------
extern void __LIB__ BIFROST_resetAnim2Frames();

// ----------------------------------------------------------------
// Reconfigure BIFROST* ENGINE to use 4 frames per animation group
// ----------------------------------------------------------------
extern void __LIB__ BIFROST_resetAnim4Frames();

// ----------------------------------------------------------------
// Advanced conversions
// ----------------------------------------------------------------
#define PX2LIN(px)              (((px)+1)<<4)
#define PX2ROW(px)              (((px)<<1)+1)

#define ROW2LIN(row)            (((row)+1)<<3)
#define ROW2PX_UP(row)          ((row)>>1)
#define ROW2PX_DOWN(row)        (((row)-1)>>1)

#define LIN2ROW_UP(lin)         (((lin)>>3)-1)
#define LIN2ROW_DOWN(lin)       (((lin)-1)>>3)
#define LIN2PX_UP(lin)          (((lin)>>4)-1)
#define LIN2PX_DOWN(lin)        (((lin)-1)>>4)

#define PY2COL(py)              (((py)<<1)+1)
#define COL2PY_LEFT(col)        (((col)-1)>>1)
#define COL2PY_RIGHT(col)       ((col)>>1)

// ----------------------------------------------------------------
// Instantly draw a multicolor tile at the specified screen position
//
// Parameters:
//     row: char row (1-17)
//     col: char column (1,3,5..17)
//     tile: tile index (0-255)
//
// WARNING: Incorrect use of this routine may cause glitches
// ----------------------------------------------------------------
extern void __LIB__ BIFROST_drawTileL(unsigned char row, unsigned char col, unsigned char tile);

// ----------------------------------------------------------------
// Instantly show/animate the multicolor tile currently stored in
// the specified tile map position
//
// Parameters:
//     row: char row (1,3,5..17)
//     col: char column (1,3,5..17)
//
// WARNING: Incorrect use of this routine may cause glitches
// ----------------------------------------------------------------
extern void __LIB__ BIFROST_showTilePosL(unsigned char row, unsigned char col);

// ----------------------------------------------------------------
// Instantly show/animate the next multicolor tile currently stored
// in the tile map position, according to a pre-established drawing
// order
//
// WARNING: Incorrect use of this routine may cause glitches
// ----------------------------------------------------------------
#define BIFROST_showNextTile()  asm(" di\n call    58727  ; BIFROST_showNextTile()\n ei\n")

// ----------------------------------------------------------------
// Instantly change the attributes in a tile area (16x16 pixels) to
// the specified value (use the same INK and PAPER values to "erase"
// a tile)
//
// Parameters:
//     row: char row (1-17)
//     col: char column (1,3,5..17)
//     attr: attribute value (0-255), INK+8*PAPER+64*BRIGHT+128*FLASH
// ----------------------------------------------------------------
extern void __LIB__ BIFROST_fillTileAttrL(unsigned char row, unsigned char col, unsigned char attr);


// ----------------------------------------------------------------
// LIBRARY OPTIMIZATIONS (developers can safely ignore this part!)
// ----------------------------------------------------------------
extern void __CALLEE__ __LIB__ BIFROST_setTile_callee(unsigned char px, unsigned char py, unsigned char tile);
#define BIFROST_setTile(px, py, tile)  BIFROST_setTile_callee(px, py, tile)
extern unsigned char __CALLEE__ __LIB__ BIFROST_getTile_callee(unsigned char px, unsigned char py);
#define BIFROST_getTile(px, py)  BIFROST_getTile_callee(px, py)
extern unsigned char __CALLEE__ __LIB__ *BIFROST_findAttrH_callee(unsigned char lin, unsigned char col);
#define BIFROST_findAttrH(lin, col)  BIFROST_findAttrH_callee(lin, col)
extern void __CALLEE__ __LIB__ BIFROST_drawTileL_callee(unsigned char row, unsigned char col, unsigned char tile);
#define BIFROST_drawTileL(row, col, tile)  BIFROST_drawTileL_callee(row, col, tile)
extern void __CALLEE__ __LIB__ BIFROST_showTilePosL_callee(unsigned char row, unsigned char col);
#define BIFROST_showTilePosL(row, col)  BIFROST_showTilePosL_callee(row, col)
extern void __CALLEE__ __LIB__ BIFROST_fillTileAttrL_callee(unsigned char row, unsigned char col, unsigned char attr);
#define BIFROST_fillTileAttrL(row, col, attr)  BIFROST_fillTileAttrL_callee(row, col, attr)

#endif
